/*
 * Copyright (c) 2005,2015 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#ifndef _CANMOTORS_H
#define _CANMOTORS_H

#ifdef __cplusplus
extern "C" {
#endif

/* Identifiers of motors */
typedef enum CAN_MOTOR_ID {
	CAN_MOTOR_LEFT,
	CAN_MOTOR_RIGHT
} CAN_MOTOR_ID;

#define CAN_MOTORS_N_MOTORS 2			/* total number of motors */

/* motor state */
typedef enum CAN_MOTOR_STATE {
	ST_PRE_INITIALIZED,
	ST_OPERATION_ENABLED,
	ST_OPERATION_DISABLED,
	ST_MOTOR_FAILURE
} CAN_MOTOR_STATE;


struct CAN_HARMONICA_STR;

/* Structure holding both controllers, with one mutex */
typedef struct CAN_MOTORS_STR {
	pthread_mutex_t *mutex;
	int status;
	struct CAN_HARMONICA_STR *motor[CAN_MOTORS_N_MOTORS];
} CAN_MOTORS_STR;

extern CAN_MOTORS_STR *canMotorsInit(int);
extern void canMotorsEnd(CAN_MOTORS_STR *);
extern int canMotorsStart(NEO_STR *);
extern int canMotorsReset(NEO_STR *);

extern bool canMotorsDecode(CAN_MOTORS_STR *, CAN_MSG *);

extern void canMotorsSetVelGearRadS(NEO_STR *, double, double);

extern void canMotorsGetGearPosVelRadS(NEO_STR *, double *, double *, 
    double *, double *);
extern void canMotorsGetGearDeltaPosVelRadS(NEO_STR *, double *, double *, 
    double *, double *);
extern CAN_MOTOR_STATE canMotorsState(NEO_STR *, CAN_MOTOR_ID);
extern void canMotorsShow(NEO_STR *);

#ifdef __cplusplus
}
#endif
#endif
