/*
 * Copyright (c) 2005-2015 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <err.h>
#include <errno.h>
#include <fcntl.h>
#include <pthread.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include <neobotix/socketcan.h>
#include <neobotix/neo.h>
#include <neobotix/canMotors.h>
#include "../src/canIds.h"
#include "../src/driveParam.h"
#include "../src/harmonica.h"

int canfd = -1;

static void*
harmonicaReadTask(void *arg)
{
        CAN_HARMONICA_STR *h = (CAN_HARMONICA_STR *)arg;
	CAN_MSG m;
	int n;
	bool decoded = false;

	do {
		n = socketcanReceiveMsgWait(canfd, &m, -1);
		decoded = harmonicaDecode(h, &m);
		if (!decoded) {
			fprintf(stderr, "%s: undecoded packet\n", __func__);
			canMsgShow(stderr, &m);
		}
	} while (n != -1);
	return NULL;
}


int
main(int argc, char *argv[])
{
	char *devname;
	int id = 0, result;
	CAN_HARMONICA_STR *m;
	pthread_t readTask;

	if (argc > 3) 
		devname = argv[2];
	else
		devname = "can0";
	if (argc > 2)
		id = atoi(argv[1]);

	printf("harmonicatest %s %d\n", devname, id);
	canfd = socketcanInit(devname);
	if (canfd < 0) 
		err(2, "socketcanInit(%s)", devname);
	m = harmonicaInit(canfd, id);
	if (m == NULL) {
		fprintf(stderr, "harmonicaInit error\n");
		exit(2);
	}
        pthread_create(&readTask, NULL, harmonicaReadTask, m);
	harmonicaInitCtrl(m);
	printf("-- InitCtrl done\n");
	// harmonicaStartWatchdog(m, true);
	result = harmonicaStart(m);
	printf("-- harmonicatStart done %d\n", result);
	// harmonicaSetGearVelRadS(h, 0.0);
	harmonicaShow(m);
	if (result == -1) {
		harmonicaEnd(m);
		close(canfd);
		exit(2);
	}
	sleep(1);
	printf("-- stopping motor\n");
	harmonicaStop(m);
	harmonicaRequestStatus(m);
	usleep(100000);
	harmonicaShow(m);
	harmonicaEnd(m);
	close(canfd);
	exit(0);
}
